import _regeneratorRuntime from "@babel/runtime/regenerator";
import "core-js/modules/es6.promise";
import "core-js/modules/web.dom.iterable";
import "core-js/modules/es6.array.iterator";
import "core-js/modules/es6.string.iterator";
import _objectWithoutProperties from "@babel/runtime/helpers/objectWithoutProperties";
import "regenerator-runtime/runtime";
import _asyncToGenerator from "@babel/runtime/helpers/asyncToGenerator";
import _objectSpread from "@babel/runtime/helpers/objectSpread";

/**
 * @license
 * Copyright © 2017-2019 Moov Corporation.  All rights reserved.
 */
import fetch from 'cross-fetch';
import qs from 'qs';
var imageService = 'image.moovweb.net';
var defaultOptions = {
  quality: 75,
  preventLayoutInstability: false
  /**
   * Transforms source to come from the Moovweb Image Optimizer
   * @param {String} img         Image source URL
   * @param {Object} options     Transformation options
   * @returns {String}           Optimized image URL
   */

};

function transformSource(img, options) {
  var query = qs.stringify(_objectSpread({}, options, {
    img: img
  }));
  return "https://opt.moovweb.net/?".concat(query);
}
/**
 * Creates a promise which fetches the image size as json
 * @param {String} src URL to image
 */


function getSize(src) {
  var url = "https://".concat(imageService, "/size?url=").concat(encodeURIComponent(src));
  return fetch(url).then(function (res) {
    return res.json();
  });
}
/**
 * Add optimization functions as plugins to cheerio
 * @param {CheerioDocument} $ A preloaded Cheerio document
 */


export default function addOptimizers($) {
  /**
   * Transforms image sources within the given elements to come from the Moovweb Image Optimizer
   * @param {Array} elements     Cheerio elements
   * @param {Object} options     Transformation options
   * @returns {String}           Optimized HTML
   */
  function optimizeImages() {
    return _optimizeImages.apply(this, arguments);
  }

  function _optimizeImages() {
    _optimizeImages = _asyncToGenerator(
    /*#__PURE__*/
    _regeneratorRuntime.mark(function _callee() {
      var options,
          preventLayoutInstability,
          transformationOptions,
          sources,
          sizes,
          _args = arguments;
      return _regeneratorRuntime.wrap(function _callee$(_context) {
        while (1) {
          switch (_context.prev = _context.next) {
            case 0:
              options = _args.length > 0 && _args[0] !== undefined ? _args[0] : defaultOptions;
              preventLayoutInstability = options.preventLayoutInstability, transformationOptions = _objectWithoutProperties(options, ["preventLayoutInstability"]);
              sources = [];
              $(this).each(function () {
                sources.push($(this).attr('src'));
              });

              if (!preventLayoutInstability) {
                _context.next = 8;
                break;
              }

              _context.next = 7;
              return Promise.all(sources.map(getSize));

            case 7:
              sizes = _context.sent;

            case 8:
              $(this).each(function (index, el) {
                var $img = $(el);
                var src = $img.attr('src');
                $img.attr('src', transformSource(src, transformationOptions));

                if (preventLayoutInstability) {
                  var _sizes$index = sizes[index],
                      width = _sizes$index.width,
                      height = _sizes$index.height;
                  $img.attr('width', width);
                  $img.attr('height', height);
                }
              });

            case 9:
            case "end":
              return _context.stop();
          }
        }
      }, _callee, this);
    }));
    return _optimizeImages.apply(this, arguments);
  }

  function lazyLoadImages() {
    $(this).each(function () {
      var $img = $(this);
      $img.attr('data-rsf-lazy', true);
      $img.attr('data-src', $img.attr('src'));
      $img.removeAttr('src');
    });
  } // Add plugins to loaded document


  $.prototype.optimizeImages = optimizeImages;
  $.prototype.lazyLoadImages = lazyLoadImages;
}
//# sourceMappingURL=optimizeImages.js.map